@@ -5,6 +5,53 @@ class TestExasol(Validator):
     dialect = "exasol"
     maxDiff = None
 
+    def test_exasol(self):
+        self.validate_identity(
+            "SELECT 1 AS [x]",
+            'SELECT 1 AS "x"',
+        )
+        self.validate_identity("SYSTIMESTAMP", "SYSTIMESTAMP()")
+        self.validate_identity("SELECT SYSTIMESTAMP()")
+        self.validate_identity("SELECT SYSTIMESTAMP(6)")
+
+    def test_qualify_unscoped_star(self):
+        self.validate_all(
+            "SELECT TEST.*, 1 FROM TEST",
+            read={
+                "": "SELECT *, 1 FROM TEST",
+            },
+        )
+        self.validate_identity(
+            "SELECT t.*, 1 FROM t",
+        )
+        self.validate_identity(
+            "SELECT t.* FROM t",
+        )
+        self.validate_identity(
+            "SELECT * FROM t",
+        )
+        self.validate_identity(
+            "WITH t AS (SELECT 1 AS x) SELECT t.*, 3 FROM t",
+        )
+        self.validate_all(
+            "WITH t1 AS (SELECT 1 AS c1), t2 AS (SELECT 2 AS c2) SELECT t1.*, t2.*, 3 FROM t1, t2",
+            read={
+                "": "WITH t1 AS (SELECT 1 AS c1), t2 AS (SELECT 2 AS c2) SELECT *, 3 FROM t1, t2",
+            },
+        )
+        self.validate_all(
+            'SELECT "A".*, "B".*, 3 FROM "A" JOIN "B" ON 1 = 1',
+            read={
+                "": 'SELECT *, 3 FROM "A" JOIN "B" ON 1=1',
+            },
+        )
+        self.validate_all(
+            "SELECT s.*, q.*, 7 FROM (SELECT 1 AS x) AS s CROSS JOIN (SELECT 2 AS y) AS q",
+            read={
+                "": "SELECT *, 7 FROM (SELECT 1 AS x) s CROSS JOIN (SELECT 2 AS y) q",
+            },
+        )
+
     def test_type_mappings(self):
         self.validate_identity("CAST(x AS BLOB)", "CAST(x AS VARCHAR)")
         self.validate_identity("CAST(x AS LONGBLOB)", "CAST(x AS VARCHAR)")
@@ -13,7 +60,10 @@ class TestExasol(Validator):
         self.validate_identity("CAST(x AS MEDIUMTEXT)", "CAST(x AS VARCHAR)")
         self.validate_identity("CAST(x AS TINYBLOB)", "CAST(x AS VARCHAR)")
         self.validate_identity("CAST(x AS TINYTEXT)", "CAST(x AS VARCHAR)")
-        self.validate_identity("CAST(x AS TEXT)", "CAST(x AS VARCHAR)")
+        self.validate_identity("CAST(x AS TEXT)", "CAST(x AS LONG VARCHAR)")
+        self.validate_identity(
+            "SELECT CAST((CAST(202305 AS INT) - 100) AS LONG VARCHAR) AS CAL_YEAR_WEEK_ADJUSTED"
+        )
         self.validate_identity("CAST(x AS VARBINARY)", "CAST(x AS VARCHAR)")
         self.validate_identity("CAST(x AS VARCHAR)", "CAST(x AS VARCHAR)")
         self.validate_identity("CAST(x AS CHAR)", "CAST(x AS CHAR)")
@@ -209,6 +259,15 @@ class TestExasol(Validator):
             },
         )
 
+        self.validate_all(
+            "SELECT a, b, rank(b) OVER (ORDER BY b) FROM (VALUES ('A1', 2), ('A1', 1), ('A2', 3), ('A1', 1)) AS tab(a, b)",
+            write={
+                "exasol": "SELECT a, b, RANK() OVER (ORDER BY b) FROM (VALUES ('A1', 2), ('A1', 1), ('A2', 3), ('A1', 1)) AS tab(a, b)",
+                "databricks": "SELECT a, b, RANK(b) OVER (ORDER BY b NULLS LAST) FROM VALUES ('A1', 2), ('A1', 1), ('A2', 3), ('A1', 1) AS tab(a, b)",
+                "spark": "SELECT a, b, RANK(b) OVER (ORDER BY b NULLS LAST) FROM VALUES ('A1', 2), ('A1', 1), ('A2', 3), ('A1', 1) AS tab(a, b)",
+            },
+        )
+
     def test_stringFunctions(self):
         self.validate_identity(
             "TO_CHAR(CAST(TO_DATE(date, 'YYYYMMDD') AS TIMESTAMP), 'DY') AS day_of_week"
@@ -224,6 +283,20 @@ class TestExasol(Validator):
         )
         self.validate_identity("SELECT TO_CHAR(12345.6789) AS TO_CHAR")
         self.validate_identity("SELECT TO_CHAR(-12345.67890, '000G000G000D000000MI') AS TO_CHAR")
+
+        self.validate_identity(
+            "SELECT id, department, hire_date, GROUP_CONCAT(id ORDER BY hire_date SEPARATOR ',') OVER (PARTITION BY department rows between 1 preceding and 1 following) GROUP_CONCAT_RESULT from employee_table ORDER BY department, hire_date",
+            "SELECT id, department, hire_date, LISTAGG(id, ',') WITHIN GROUP (ORDER BY hire_date) OVER (PARTITION BY department rows BETWEEN 1 preceding AND 1 following) AS GROUP_CONCAT_RESULT FROM employee_table ORDER BY department, hire_date",
+        )
+        self.validate_all(
+            "GROUP_CONCAT(DISTINCT x ORDER BY y DESC)",
+            write={
+                "exasol": "LISTAGG(DISTINCT x, ',') WITHIN GROUP (ORDER BY y DESC)",
+                "mysql": "GROUP_CONCAT(DISTINCT x ORDER BY y DESC SEPARATOR ',')",
+                "tsql": "STRING_AGG(x, ',') WITHIN GROUP (ORDER BY y DESC)",
+                "databricks": "LISTAGG(DISTINCT x, ',') WITHIN GROUP (ORDER BY y DESC)",
+            },
+        )
         self.validate_all(
             "EDIT_DISTANCE(col1, col2)",
             read={
@@ -266,10 +339,10 @@ class TestExasol(Validator):
                 "SELECT TO_CHAR(CAST('1999-12-31' AS DATE)) AS TO_CHAR",
                 write={
                     "exasol": "SELECT TO_CHAR(CAST('1999-12-31' AS DATE)) AS TO_CHAR",
-                    "redshift": "SELECT TO_CHAR(CAST('1999-12-31' AS DATE)) AS TO_CHAR",
                     "presto": "SELECT DATE_FORMAT(CAST('1999-12-31' AS DATE)) AS TO_CHAR",
                     "oracle": "SELECT TO_CHAR(CAST('1999-12-31' AS DATE)) AS TO_CHAR",
-                    "postgres": "SELECT TO_CHAR(CAST('1999-12-31' AS DATE)) AS TO_CHAR",
+                    "redshift": "SELECT CAST(CAST('1999-12-31' AS DATE) AS VARCHAR(MAX)) AS TO_CHAR",
+                    "postgres": "SELECT CAST(CAST('1999-12-31' AS DATE) AS TEXT) AS TO_CHAR",
                 },
                 read={
                     "exasol": "SELECT TO_CHAR(DATE '1999-12-31') AS TO_CHAR",
@@ -295,6 +368,31 @@ class TestExasol(Validator):
                 "presto": r"SELECT REGEXP_EXTRACT('My mail address is my_mail@yahoo.com', '(?i)[a-z0-9._%+-]+@[a-z0-9.-]+\.[a-z]{2,4}') AS EMAIL",
             },
         )
+        self.validate_all(
+            "SELECT SUBSTR('www.apache.org', 1, NVL(NULLIF(INSTR('www.apache.org', '.', 1, 2), 0) - 1, LENGTH('www.apache.org')))",
+            read={
+                "databricks": "SELECT substring_index('www.apache.org', '.', 2)",
+            },
+        )
+
+        self.validate_all(
+            "SELECT SUBSTR('555A66A777', 1, NVL(NULLIF(INSTR('555A66A777', 'a', 1, 2), 0) - 1, LENGTH('555A66A777')))",
+            read={
+                "databricks": "SELECT substring_index('555A66A777' COLLATE UTF8_BINARY, 'a', 2)",
+            },
+        )
+        self.validate_all(
+            "SELECT SUBSTR('555A66A777', 1, NVL(NULLIF(INSTR(LOWER('555A66A777'), 'a', 1, 2), 0) - 1, LENGTH('555A66A777')))",
+            read={
+                "databricks": "SELECT substring_index('555A66A777' COLLATE UTF8_LCASE, 'a', 2)",
+            },
+        )
+        self.validate_all(
+            "SELECT SUBSTR('A|a|A', 1, NVL(NULLIF(INSTR(LOWER('A|a|A'), LOWER('A'), 1, 2), 0) - 1, LENGTH('A|a|A')))",
+            read={
+                "databricks": "SELECT substring_index('A|a|A' COLLATE UTF8_LCASE, 'A' COLLATE UTF8_LCASE, 2)",
+            },
+        )
 
     def test_datetime_functions(self):
         formats = {
@@ -320,6 +418,7 @@ class TestExasol(Validator):
         )
         self.validate_identity("SELECT TO_DATE('31-DECEMBER-1999', 'DD-MONTH-YYYY') AS TO_DATE")
         self.validate_identity("SELECT TO_DATE('31-DEC-1999', 'DD-MON-YYYY') AS TO_DATE")
+        self.validate_identity("SELECT WEEKOFYEAR('2024-05-22')", "SELECT WEEK('2024-05-22')")
 
         for fmt, alias in formats.items():
             with self.subTest(f"Testing TO_CHAR with format '{fmt}'"):
@@ -327,6 +426,16 @@ class TestExasol(Validator):
                     f"SELECT TO_CHAR(CAST('2024-07-08 13:45:00' AS TIMESTAMP), '{fmt}') AS {alias}"
                 )
 
+        self.validate_all(
+            "SELECT TO_CHAR(CAST('2024-07-08 13:45:00' AS TIMESTAMP), 'DY')",
+            write={
+                "exasol": "SELECT TO_CHAR(CAST('2024-07-08 13:45:00' AS TIMESTAMP), 'DY')",
+                "oracle": "SELECT TO_CHAR(CAST('2024-07-08 13:45:00' AS TIMESTAMP), 'DY')",
+                "postgres": "SELECT TO_CHAR(CAST('2024-07-08 13:45:00' AS TIMESTAMP), 'TMDy')",
+                "databricks": "SELECT DATE_FORMAT(CAST('2024-07-08 13:45:00' AS TIMESTAMP), 'EEE')",
+            },
+        )
+
         self.validate_all(
             "TO_DATE(x, 'YYYY-MM-DD')",
             write={
@@ -383,26 +492,98 @@ class TestExasol(Validator):
             "SELECT CAST(CAST(CURRENT_TIMESTAMP() AS TIMESTAMP) AT TIME ZONE 'CET' AS DATE) - 1",
             "SELECT CAST(CONVERT_TZ(CAST(CURRENT_TIMESTAMP() AS TIMESTAMP), 'UTC', 'CET') AS DATE) - 1",
         )
+        units = ["MM", "QUARTER", "WEEK", "MINUTE", "YEAR"]
+        for unit in units:
+            with self.subTest(f"Testing DATE_TRUNC with format '{unit}'"):
+                self.validate_all(
+                    f"SELECT TRUNC(CAST('2006-12-31' AS DATE), '{unit}') AS TRUNC",
+                    write={
+                        "exasol": f"SELECT DATE_TRUNC('{unit}', DATE '2006-12-31') AS TRUNC",
+                        "presto": f"SELECT DATE_TRUNC('{unit}', CAST('2006-12-31' AS DATE)) AS TRUNC",
+                        "databricks": f"SELECT TRUNC(CAST('2006-12-31' AS DATE), '{unit}') AS TRUNC",
+                    },
+                )
 
+                self.validate_all(
+                    f"SELECT DATE_TRUNC('{unit}', TIMESTAMP '2006-12-31T23:59:59') DATE_TRUNC",
+                    write={
+                        "exasol": f"SELECT DATE_TRUNC('{unit}', TIMESTAMP '2006-12-31 23:59:59') AS DATE_TRUNC",
+                        "presto": f"SELECT DATE_TRUNC('{unit}', CAST('2006-12-31T23:59:59' AS TIMESTAMP)) AS DATE_TRUNC",
+                        "databricks": f"SELECT DATE_TRUNC('{unit}', CAST('2006-12-31T23:59:59' AS TIMESTAMP)) AS DATE_TRUNC",
+                    },
+                )
+                self.validate_all(
+                    f"SELECT DATE_TRUNC('{unit}', CURRENT_TIMESTAMP) DATE_TRUNC",
+                    write={
+                        "exasol": f"SELECT DATE_TRUNC('{unit}', CURRENT_TIMESTAMP()) AS DATE_TRUNC",
+                        "presto": f"SELECT DATE_TRUNC('{unit}', CURRENT_TIMESTAMP) AS DATE_TRUNC",
+                        "databricks": f"SELECT DATE_TRUNC('{unit}', CURRENT_TIMESTAMP()) AS DATE_TRUNC",
+                    },
+                )
+
+        from sqlglot.dialects.exasol import DATE_UNITS
+
+        for unit in DATE_UNITS:
+            with self.subTest(f"Testing ADD_{unit}S"):
+                self.validate_all(
+                    f"SELECT ADD_{unit}S(DATE '2000-02-28', 1)",
+                    write={
+                        "exasol": f"SELECT ADD_{unit}S(CAST('2000-02-28' AS DATE), 1)",
+                        "bigquery": f"SELECT DATE_ADD(CAST('2000-02-28' AS DATE), INTERVAL 1 {unit})",
+                        "duckdb": f"SELECT CAST('2000-02-28' AS DATE) + INTERVAL 1 {unit}",
+                        "presto": f"SELECT DATE_ADD('{unit}', 1, CAST('2000-02-28' AS DATE))",
+                        "redshift": f"SELECT DATEADD({unit}, 1, CAST('2000-02-28' AS DATE))",
+                        "snowflake": f"SELECT DATEADD({unit}, 1, CAST('2000-02-28' AS DATE))",
+                        "tsql": f"SELECT DATEADD({unit}, 1, CAST('2000-02-28' AS DATE))",
+                    },
+                )
+
+                self.validate_all(
+                    f"SELECT ADD_{unit}S('2000-02-28', -'1')",
+                    read={
+                        "sqlite": f"SELECT DATE_SUB('2000-02-28', INTERVAL 1 {unit})",
+                        "bigquery": f"SELECT DATE_SUB('2000-02-28', INTERVAL 1 {unit})",
+                        "presto": f"SELECT DATE_SUB('2000-02-28', INTERVAL 1 {unit})",
+                        "redshift": f"SELECT DATE_SUB('2000-02-28', INTERVAL 1 {unit})",
+                        "snowflake": f"SELECT DATE_SUB('2000-02-28', INTERVAL 1 {unit})",
+                        "tsql": f"SELECT DATE_SUB('2000-02-28', INTERVAL 1 {unit})",
+                    },
+                )
+
+                self.validate_all(
+                    "SELECT CAST(ADD_DAYS(ADD_MONTHS(DATE_TRUNC('MONTH', DATE '2008-11-25'), 1), -1) AS DATE)",
+                    read={
+                        "snowflake": "SELECT LAST_DAY(CAST('2008-11-25' AS DATE), MONTH)",
+                        "databricks": "SELECT LAST_DAY('2008-11-25')",
+                        "spark": "SELECT LAST_DAY(CAST('2008-11-25' AS DATE))",
+                        "presto": "SELECT LAST_DAY_OF_MONTH(CAST('2008-11-25' AS DATE))",
+                    },
+                )
+
+            with self.subTest(f"Testing {unit}S_BETWEEN"):
+                self.validate_all(
+                    f"SELECT {unit}S_BETWEEN(TIMESTAMP '2000-02-28 00:00:00', CURRENT_TIMESTAMP)",
+                    write={
+                        "exasol": f"SELECT {unit}S_BETWEEN(CAST('2000-02-28 00:00:00' AS TIMESTAMP), CURRENT_TIMESTAMP())",
+                        "bigquery": f"SELECT DATE_DIFF(CAST('2000-02-28 00:00:00' AS DATETIME), CURRENT_TIMESTAMP(), {unit})",
+                        "duckdb": f"SELECT DATE_DIFF('{unit}', CURRENT_TIMESTAMP, CAST('2000-02-28 00:00:00' AS TIMESTAMP))",
+                        "presto": f"SELECT DATE_DIFF('{unit}', CURRENT_TIMESTAMP, CAST('2000-02-28 00:00:00' AS TIMESTAMP))",
+                        "redshift": f"SELECT DATEDIFF({unit}, GETDATE(), CAST('2000-02-28 00:00:00' AS TIMESTAMP))",
+                        "snowflake": f"SELECT DATEDIFF({unit}, CURRENT_TIMESTAMP(), CAST('2000-02-28 00:00:00' AS TIMESTAMP))",
+                        "tsql": f"SELECT DATEDIFF({unit}, GETDATE(), CAST('2000-02-28 00:00:00' AS DATETIME2))",
+                    },
+                )
         self.validate_all(
-            "SELECT TRUNC(CAST('2006-12-31' AS DATE), 'MM') AS TRUNC",
+            "SELECT quarter('2016-08-31')",
             write={
-                "exasol": "SELECT TRUNC(CAST('2006-12-31' AS DATE), 'MM') AS TRUNC",
-                "presto": "SELECT DATE_TRUNC('MM', CAST('2006-12-31' AS DATE)) AS TRUNC",
-                "databricks": "SELECT TRUNC(CAST('2006-12-31' AS DATE), 'MM') AS TRUNC",
-            },
-        )
-        self.validate_all(
-            "SELECT DATE_TRUNC('minute', TIMESTAMP '2006-12-31 23:59:59') DATE_TRUNC",
-            write={
-                "exasol": "SELECT DATE_TRUNC('MINUTE', CAST('2006-12-31 23:59:59' AS TIMESTAMP)) AS DATE_TRUNC",
-                "presto": "SELECT DATE_TRUNC('MINUTE', CAST('2006-12-31 23:59:59' AS TIMESTAMP)) AS DATE_TRUNC",
-                "databricks": "SELECT DATE_TRUNC('MINUTE', CAST('2006-12-31 23:59:59' AS TIMESTAMP)) AS DATE_TRUNC",
+                "exasol": "SELECT CEIL(MONTH(TO_DATE('2016-08-31'))/3)",
+                "databricks": "SELECT QUARTER('2016-08-31')",
             },
         )
 
     def test_number_functions(self):
         self.validate_identity("SELECT TRUNC(123.456, 2) AS TRUNC")
+        self.validate_identity("SELECT DIV(1234, 2) AS DIV")
 
     def test_scalar(self):
         self.validate_all(
@@ -509,3 +690,90 @@ class TestExasol(Validator):
                 "trino": "SHA512(x)",
             },
         )
+        self.validate_all(
+            "SELECT NULLIFZERO(1) NIZ1",
+            write={
+                "exasol": "SELECT IF 1 = 0 THEN NULL ELSE 1 ENDIF AS NIZ1",
+                "snowflake": "SELECT IFF(1 = 0, NULL, 1) AS NIZ1",
+                "sqlite": "SELECT IIF(1 = 0, NULL, 1) AS NIZ1",
+                "presto": "SELECT IF(1 = 0, NULL, 1) AS NIZ1",
+                "spark": "SELECT IF(1 = 0, NULL, 1) AS NIZ1",
+                "hive": "SELECT IF(1 = 0, NULL, 1) AS NIZ1",
+                "duckdb": "SELECT CASE WHEN 1 = 0 THEN NULL ELSE 1 END AS NIZ1",
+            },
+        )
+        self.validate_all(
+            "SELECT ZEROIFNULL(NULL) NIZ1",
+            write={
+                "exasol": "SELECT IF NULL IS NULL THEN 0 ELSE NULL ENDIF AS NIZ1",
+                "snowflake": "SELECT IFF(NULL IS NULL, 0, NULL) AS NIZ1",
+                "sqlite": "SELECT IIF(NULL IS NULL, 0, NULL) AS NIZ1",
+                "presto": "SELECT IF(NULL IS NULL, 0, NULL) AS NIZ1",
+                "spark": "SELECT IF(NULL IS NULL, 0, NULL) AS NIZ1",
+                "hive": "SELECT IF(NULL IS NULL, 0, NULL) AS NIZ1",
+                "duckdb": "SELECT CASE WHEN NULL IS NULL THEN 0 ELSE NULL END AS NIZ1",
+            },
+        )
+        self.validate_identity(
+            "SELECT name, age, IF age < 18 THEN 'underaged' ELSE 'adult' ENDIF AS LEGALITY FROM persons"
+        )
+
+        self.validate_identity("SELECT HASHTYPE_MD5(a, b, c, d)")
+
+    def test_odbc_date_literals(self):
+        self.validate_identity("SELECT {d'2024-01-01'}", "SELECT TO_DATE('2024-01-01')")
+        self.validate_identity(
+            "SELECT {ts'2024-01-01 12:00:00'}",
+            "SELECT TO_TIMESTAMP('2024-01-01 12:00:00')",
+        )
+
+    def test_local_prefix_for_alias(self):
+        self.validate_identity(
+            'SELECT ID FROM local WHERE "LOCAL".ID IS NULL',
+            'SELECT ID FROM "LOCAL" WHERE "LOCAL".ID IS NULL',
+        )
+        self.validate_identity(
+            'SELECT YEAR(a_date) AS "a_year" FROM MY_SUMMARY_TABLE GROUP BY LOCAL."a_year"',
+        )
+        self.validate_identity(
+            'SELECT a_year AS a_year FROM "LOCAL" GROUP BY "LOCAL".a_year',
+        )
+
+        test_cases = [
+            (
+                "GROUP BY alias",
+                "SELECT YEAR(a_date) AS a_year FROM my_table GROUP BY LOCAL.a_year",
+                "SELECT YEAR(a_date) AS a_year FROM my_table GROUP BY a_year",
+            ),
+            (
+                "HAVING alias",
+                "SELECT SUM(amount) AS total FROM my_table HAVING LOCAL.total > 10000",
+                "SELECT SUM(amount) AS total FROM my_table HAVING total > 10000",
+            ),
+            (
+                "WHERE alias",
+                "SELECT YEAR(a_date) AS a_year FROM my_table WHERE LOCAL.a_year > 2020",
+                "SELECT YEAR(a_date) AS a_year FROM my_table WHERE a_year > 2020",
+            ),
+            (
+                "Multiple aliases",
+                "SELECT YEAR(a_date) AS a_year, MONTH(a_date) AS a_month FROM my_table WHERE LOCAL.a_year > 2020 AND LOCAL.a_month < 6",
+                "SELECT YEAR(a_date) AS a_year, MONTH(a_date) AS a_month FROM my_table WHERE a_year > 2020 AND a_month < 6",
+            ),
+            (
+                "Select list aliases",
+                "SELECT YR AS THE_YEAR, ID AS YR, LOCAL.THE_YEAR + 1 AS NEXT_YEAR FROM my_table",
+                "SELECT YR AS THE_YEAR, ID AS YR, THE_YEAR + 1 AS NEXT_YEAR FROM my_table",
+            ),
+            (
+                "Select list aliases without Local keyword",
+                "SELECT YEAR(CURRENT_DATE) AS current_year, LOCAL.current_year + 1 AS next_year",
+                "SELECT YEAR(CURRENT_DATE) AS current_year, current_year + 1 AS next_year",
+            ),
+        ]
+        for title, exasol_sql, dbx_sql in test_cases:
+            with self.subTest(clause=title):
+                self.validate_all(
+                    exasol_sql,
+                    write={"exasol": exasol_sql, "databricks": dbx_sql},
+                )
