@@ -949,3 +949,7 @@ SELECT analyze FROM (SELECT 1 AS analyze)
 SELECT 'Ac' ILIKE 'a%c' ESCAPE NULL
 SELECT CURRENT_DATABASE()
 SELECT CURRENT_SCHEMAS(arg_bool)
+SELECT UNIFORM(1, 10, 5)
+SELECT UNIFORM(1, 10)
+SELECT CURRENT_TIMEZONE()
+SELECT NUMRANGE(1.1, 2.2) -|- NUMRANGE(2.2, 3.3)
\ No newline at end of file
